@@ -4,10 +4,15 @@
 #define TEXT 260
 #define NUMBER 261
 #define MACADDR 262
+#define MACADDR_PARTIAL 263
 typedef union {
 		long number;
 		char *name;
 		char *text;
 		char mac_address[6];
+		struct partial_mac {
+			char mac_address[6];
+			int mac_len;
+		} pm;
 	} YYSTYPE;
 extern YYSTYPE yylval;
